using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using InstrumentDrivers;

namespace RSSMA_PM_Example
{
    public partial class dialog : Form
    {
        protected string _strGen1Freq = "1e+3", _strGen1Deviation = "10";
        protected string _strGen2Freq = "1e+3", _strGen2Rise = "1e-6", _strGen2High = "1e-6", _strGen2Fall = "1e-6", _strGen2Low = "1e-6";
        protected int _nGen2Shape = 0;
        protected string _strNoiseBandWidth = "100e+3";
        protected int _nNoiseDistribution = 0;
        protected bool _bNoiseBandLimitation;

        public dialog()
        {
            InitializeComponent();
        }

        private void dialog_Load(object sender, EventArgs e)
        {
            PMSource.SelectedIndex = 0;
            PMMode.SelectedIndex = 0;
        }
        
        private void ExitBTN_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void ApplyBTN_Click(object sender, EventArgs e)
        {
            rssma instrument;

            try
            {
                UseWaitCursor = true;
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor;
                instrument = new rssma(ResourceDescriptor.Text, IDQuery.Checked, ResetDevice.Checked);

                double dGen1Deviation = Convert.ToDouble(_strGen1Deviation);
                double dGen1Freq = Convert.ToDouble(_strGen1Freq);
                double dGen2Freq = Convert.ToDouble(_strGen2Freq);
                double dGen2Fall = Convert.ToDouble(_strGen2Fall);
                double dGen2Rise = Convert.ToDouble(_strGen2Rise);
                double dGen2High = Convert.ToDouble(_strGen2High);
                double dGen2Low = Convert.ToDouble(_strGen2Low);

                instrument.confRFFreq(Decimal.ToDouble(RFFrequency.Value), 0.0, false, 0.0);
                instrument.confRFLevel(Decimal.ToDouble(RFLevel.Value), 0.0, 30.0, false, 0.0, 
                    rssmaConstants.OutputAmodeAuto, rssmaConstants.PowerLevelRclExclude);

                instrument.confPM(PMState.Checked, rssmaConstants.FmSourceInternal,
                                       PMMode.SelectedIndex, Decimal.ToDouble(PMDeviation.Value), Decimal.ToDouble(PMDeviation.Value), 0.0,
                                       rssmaConstants.ExternalCouplingAc,
                                       rssmaConstants.ExtInputImpedanceG50,
                                       rssmaConstants.FmExternalDigitalFormatBoffset);  


                int nPMSource = PMSource.SelectedIndex;
                if (nPMSource != 0)
                {
                    instrument.confPMInternal(nPMSource - 1, dGen1Deviation, dGen1Freq,
                                               _nGen2Shape, dGen2Freq);


                    if (nPMSource == 1 || nPMSource == 3 || nPMSource == 5)
                        instrument.confLFFreq(true, 1, dGen1Freq, 1.0);

                    if (nPMSource == 2 || nPMSource == 3 || nPMSource == 6)
                    {
                        instrument.confLFFreq(true, 2, dGen2Freq, 1.0);
                        instrument.SetLFGenShape(_nGen2Shape);

                        if (_nGen2Shape == 3) // trapeze
                        {
                            instrument.SetLFGenShapeTrapeze(rssmaConstants.LfoutputShapeTrapezeFall, dGen2Fall);
                            instrument.SetLFGenShapeTrapeze(rssmaConstants.LfoutputShapeTrapezeRise, dGen2Rise);
                            instrument.SetLFGenShapeTrapeze(rssmaConstants.LfoutputShapeTrapezeHigh, dGen2High);
                            instrument.SetLFGenShapeTrapeze(rssmaConstants.LfoutputShapeTrapezeLow, dGen2Low);
                        }
                    }
                    if (nPMSource == 4 || nPMSource == 5 || nPMSource == 6)
                    {
                        double dNoiseBandWidth = Convert.ToDouble(_strNoiseBandWidth);
                        instrument.confNoise(_bNoiseBandLimitation, dNoiseBandWidth, _nNoiseDistribution);
                    }
                }

                instrument.SetOutputState(RFState.Checked);
            }
            catch (Exception ex)
            {
                String message;

                message = "Instrument Status Error: ";
                message += ex.Message;
                MessageBox.Show(message);
            }
            System.Windows.Forms.Cursor.Current = Cursors.Arrow;
            UseWaitCursor = false;
        }

        private void OnGen1(object sender, EventArgs e)
        {
            DlgGen1 dlg = new DlgGen1();
            dlg.Freq = _strGen1Freq;
            dlg.Deviation = _strGen1Deviation;

            if ( dlg.ShowDialog() == DialogResult.OK )
            {
                _strGen1Freq = dlg.Freq;
                _strGen1Deviation = dlg.Deviation;
            }
        }

        private void OnGen2(object sender, EventArgs e)
        {
            DlgGen2 dlg = new DlgGen2();

            dlg.Freq = _strGen2Freq;
            dlg.Shape = _nGen2Shape;
            dlg.Rise = _strGen2Rise;
            dlg.High = _strGen2High;
            dlg.Fall = _strGen2Fall;
            dlg.Low = _strGen2Low;

            if ( dlg.ShowDialog() == DialogResult.OK )
            {
                _strGen2Freq = dlg.Freq;
                _nGen2Shape = dlg.Shape;
                _strGen2Rise = dlg.Rise;
                _strGen2High = dlg.High;
                _strGen2Fall = dlg.Fall;
                _strGen2Low = dlg.Low;
            }

        }

        private void OnNoise(object sender, EventArgs e)
        {
            DlgNoise dlg = new DlgNoise();

            dlg.Distribution = _nNoiseDistribution;
            dlg.BandLimitation = _bNoiseBandLimitation;
            dlg.BandWidth = _strNoiseBandWidth;

            if ( dlg.ShowDialog() == DialogResult.OK )
            {
                _nNoiseDistribution = dlg.Distribution;
                _bNoiseBandLimitation = dlg.BandLimitation;
                _strNoiseBandWidth = dlg.BandWidth;
            }
        }

        private void OnSourceChange(object sender, EventArgs e)
        {
            bool bGen1 = false, bGen2 = false, bNoise = false;

            switch (PMSource.SelectedIndex)
            {
                case 0: // none
                    break;
                case 1: // gen 1
                    bGen1 = true;
                    break;
                case 2: // gen 2
                    bGen2 = true;
                    break;
                case 3: // gen 1 + gen 2
                    bGen1 = bGen2 = true;
                    break;
                case 4: // noise
                    bNoise = true;
                    break;
                case 5: // gen 1 + noise
                    bGen1 = bNoise = true;
                    break;
                case 6: // gen 2 + noise
                    bGen2 = bNoise = true;
                    break;
            }

            _gen1.Enabled = bGen1;
            _gen2.Enabled = bGen2;
            _noise.Enabled = bNoise;
        }
        
    }
}